@@ -23,7 +23,7 @@
 #ifndef _UTIL_H_
 #define _UTIL_H_
 
-#include <config.h>
+#include "config.h"
 #include <stdbool.h>
 #include <string.h>
 
@@ -34,6 +34,7 @@
 
 #undef min
 #undef max
+#ifdef __GNUC__
 #define min(a,b) \
 		({ __typeof__ (a) _a = (a); \
 	          __typeof__ (b) _b = (b); \
@@ -44,6 +45,10 @@
 	          __typeof__ (b) _b = (b); \
 		_a > _b ? _a : _b; \
 		})
+#else
+#define min(a,b) ((a) > (b) ? (b) : (a))
+#define max(a,b) ((a) > (b) ? (a) : (b))
+#endif
 
 static inline bool
 startswith(const char *str, size_t len, const char *prefix, size_t plen)
