@@ -20,7 +20,7 @@
  * OF THIS SOFTWARE.
  */
 
-#include <config.h>
+#include "config.h"
 #include <fcntl.h>
 #include <poll.h>
 #include <unistd.h>
