@@ -20,8 +20,7 @@
  * OF THIS SOFTWARE.
  */
 
-#define _GNU_SOURCE
-#include <config.h>
+#include "config.h"
 
 #include <getopt.h>
 #include <limits.h>
@@ -37,7 +36,7 @@
 #include <assert.h>
 #include <linux/input.h>
 
-#include "libevdev.h"
+#include "libevdev/libevdev.h"
 
 static void
 usage(void)
