@@ -21,11 +21,8 @@
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
-#include <libevdev/libevdev.h>
 #include <sys/signalfd.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -38,6 +35,8 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "libevdev/libevdev.h"
+
 #define min(a, b) (((a) < (b)) ? (a) : (b))
 #define max(a, b) (((a) > (b)) ? (a) : (b))
 
@@ -61,12 +60,6 @@ usage(void) {
 	return 1;
 }
 
-static inline uint64_t
-tv2us(const struct timeval *tv)
-{
-	return tv->tv_sec * 1000000 + tv->tv_usec;
-}
-
 static inline double
 get_frequency(uint64_t last, uint64_t current)
 {
@@ -119,7 +112,7 @@ handle_event(struct measurements *m, const struct input_event *ev)
 		const int idle_reset = 3000000; /* us */
 		uint64_t last_us = m->us;
 
-		m->us = tv2us(&ev->time);
+		m->us = ev->input_event_sec * 1000000 + ev->input_event_usec;
 
 		/* reset after pause */
 		if (last_us + idle_reset < m->us) {
