@@ -21,11 +21,8 @@
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
-#include <libevdev/libevdev.h>
 #include <sys/signalfd.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -39,6 +36,8 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "libevdev/libevdev.h"
+
 #define min(a, b) (((a) < (b)) ? (a) : (b))
 #define max(a, b) (((a) > (b)) ? (a) : (b))
 
